<?php

namespace Drupal\entity_holder;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * Entity Holder entity helper.
 */
class EntityHolderEntityHelper {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * EntityHolderEntityHelper constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Check access to the held entity creation form.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user account.
   * @param \Drupal\entity_holder\EntityHolderInterface $entity_holder
   *   The entity holder.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   Allowed if the user has permission to create the held entity.
   */
  public function checkHeldEntityFormAccess(AccountInterface $account, EntityHolderInterface $entity_holder) {
    $held_entity_access_handler = $this->entityTypeManager->getAccessControlHandler($entity_holder->getHeldEntityTypeId());
    if ($held_entity_access_handler->createAccess($entity_holder->getHeldEntityBundle())) {
      return AccessResult::allowed();
    }

    // Deny access if the user has no permission to create the held entity.
    return AccessResult::forbidden();
  }

}
