<?php

namespace Drupal\entity_holder;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Provides a listing of entity holders.
 */
class EntityHolderListBuilder extends ConfigEntityListBuilder {

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['label'] = $this->t('Label');
    $header['id'] = $this->t('Machine name');
    $header['description'] = $this->t('Description');
    $header['path'] = $this->t('Path');
    $header['entity'] = $this->t('Held entity');
    $header['status'] = $this->t('Status');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    /** @var \Drupal\entity_holder\EntityHolderInterface $entity */
    /** @var \Drupal\Core\Entity\ContentEntityInterface $held_entity */
    $held_entity = $entity->getHeldEntity();

    $row['label'] = $entity->label();
    $row['id'] = $entity->id();
    $row['description'] = $entity->getDescription();
    $path = $entity->get('path');
    $row['path'] = Link::fromTextAndUrl($path, Url::fromUserInput($path));
    $row['entity'] = $held_entity ? $held_entity->toLink() : $this->t('- None -');
    $row['status'] = $entity->status() ? $this->t('Enabled') : $this->t('Disabled');

    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {
    /** @var \Drupal\entity_holder\EntityHolderInterface $entity */
    $operations = [];

    if (!$entity->getHeldEntity()) {
      $create_url = Url::fromRoute('entity.entity_holder.held_entity_form', ['entity_holder' => $entity->id()]);
      if ($create_url->access()) {
        $this->ensureDestination($create_url);
        // Add the create held entity operation.
        $operations = [
          'create_held_entity' => [
            'title' => $this->t('Create held entity'),
            'weight' => 0,
            'url' => $create_url,
          ],
        ];
      }
    }

    return $operations + parent::getDefaultOperations($entity);
  }

}
